(* Computes the square root of x using Heron of Alexandria's
 * algorithm (circa 100 AD).  We start with an initial (poor)
 * approximate answer that the square root is 1.0 and then
 * continue improving the guess until we're within delta of the
 * real answer.  The improvement is achieved by averaging the
 * current guess with x/guess.  The answer is accurate to within
 * delta = 0.0001. *)
let squareRoot (x : float) : float =
  (* numerical accuracy *)
  let delta = 0.0001
  in
  
  (* returns true iff the guess is good enough *)
  let goodEnough (guess : float) : bool =
    abs_float (guess *. guess -. x) < delta
  in

  (* return a better guess by averaging it with x/guess *)
  let improve (guess : float) : float =
    (guess +. x /. guess) /. 2.0
  in
      
  (* Return the square root of x, starting from an initial guess. *)
  let rec tryGuess (guess : float) : float =
    if goodEnough guess then guess
    else tryGuess (improve guess)
  in

  (* start with a guess of 1.0 *)
  tryGuess 1.0
